import Data.List
import Data.Array.Unboxed

type Matrix = UArray (Int, Int) Double

mMult :: Matrix -> Matrix -> Matrix
mMult a b = listArray (bounds a) . map mElem $ range (bounds a)
    where
        ((l, _), (h, _)) = bounds a
        mElem (i, j) = sum [(a ! (i, k)) * (b ! (k, j)) | k <- [l..h]]

idM :: Int -> Matrix
idM n = listArray ((1, 1), (n, n)) (replicate (n*n) 0) // [((i, i), 1) | i <- [1..n]]

mExp :: Int -> Matrix -> Matrix
mExp 1 m = m
mExp n m
    | even n = mMult m2 m2
    | otherwise = mMult m $ mMult m2 m2
    where
        m2 = mExp (div n 2) m

diceMatrix :: Int -> Matrix
diceMatrix n = listArray ((1, 1), (n, n)) (replicate (n*n) 0) // (compress [((i, min n j), 1/6) | i <- [1..n], j <- [i+1..i+6]])
    where
        compress = map (\es -> (fst $ head es, sum $ map snd es)) . groupBy (\a b -> fst a == fst b)

ladderMatrix :: Int -> [(Int, Int)] -> Matrix
ladderMatrix n ladders = idM n // [((i, i), 0) | i <- map fst ladders] // map (\x -> (x, 1)) ladders

createMatrix :: Int -> [(Int, Int)] -> Matrix
createMatrix n ladders = mMult (diceMatrix n) (ladderMatrix n ladders)

binSearch :: (Int -> Bool) -> Int -> Int -> Int
binSearch p a b
    | a >= b    = a
    | p m       = binSearch p a m
    | otherwise = binSearch p (m+1) b
    where
        m = (a + b) `div` 2

solve :: Int -> Double -> [(Int, Int)] -> Int
solve n p ladders = binSearch (\x -> mExp x m ! (1, n) >= p) 1 100000000
    where
        m = createMatrix n ladders

getInput :: [[String]] -> (Int, Double, [(Int, Int)])
getInput (n':p':ls') = (r*c, p, ls)
    where
        [r, c, _] = map read n'
        [p] = map read p'
        ls = map ((\[a, b] -> (a, b)) . map read) ls'

main :: IO ()
main = interact ((++"\n") . show . (\(n, p, ls) -> solve n p ls) . getInput . map words . lines)
